-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Main = DellerenAddon
local L = Main.Locale


-------------------------------------------------------------------------------
local CALL_SPELL_PRESETS = {
	painsups      = { 6940, 33206, 114030, 102342 };
	antimagiczone = { 51052  };
	ironbark      = { 102342 };
	sac           = { 6940   };
	bop           = { 1022   };
	psup          = { 33206  };
	smokebomb     = { 76577  };
	vig           = { 114030 };
}

-------------------------------------------------------------------------------
local CALL_SPELL_PRESETS_ITEMS = {
	jeeves   = { 49040 };
}

-------------------------------------------------------------------------------
local function TableContainsValue( table, value )
	for k,v in pairs(table) do
		if v == value then return k end
	end
	
	return nil
end

-------------------------------------------------------------------------------
local function PushCallSpell( spells, arg )

	local list = nil
	
	local makeitem = false
	
	if CALL_SPELL_PRESETS[arg] then
	
		list = CALL_SPELL_PRESETS[arg]
		
	elseif CALL_SPELL_PRESETS_ITEMS[arg] then
	
		list = CALL_SPELL_PRESETS_ITEMS[arg]
		makeitem = true
		
	else
	
		local spellid = tonumber( arg )
		if not spellid then
		
			Main:Print( L["Invalid spell ID: "] .. arg )
			return
		end
		
		list = { spellid }
	end
	
	for _,spell in ipairs(list) do
	
		if not TableContainsValue( spells, spell ) then
			table.insert( spells, spell )
		end
	end
	
	return makeitem
end

-------------------------------------------------------------------------------
-- Parse a call command and execute it.
--
function Main:CallCommand( args )

	if self.Query.active then return end
	
	local spells  = {}
	local players = {}
	local item    = false
	local manual  = false
	local buff    = true
	
	local mode = "SPELLS"
	
	for i = 2,#args do
		local arg = args[i]
		
		if string.sub(arg,1,1) == "-" then
			-- option
			
			if arg == "-s" then
				mode = "SPELLS"
			elseif arg == "-p" then
				mode = "PLAYERS"
			elseif arg == "-i" then
				item = true
			elseif arg == "-m" then
				manual = true
			elseif arg == "-c" then
				buff = false
			else
				Main:Print( L["Unknown option: "] .. arg )
			end
			
		else
		
			if mode == "SPELLS" then
			
				if PushCallSpell( spells, arg ) then
					item = true
				end
				
			elseif mode == "PLAYERS" then
			
				table.insert( players, string.lower(arg) )
			end
		end
	end
	
	if #spells == 0 then
		Main:Print( L["No spell IDs given!"] )
		return
	end
	 
	self.Query:Start( spells, item, buff, manual, players )
end

-------------------------------------------------------------------------------
function Main:WhoCommand()
	
	local players_with    = {}
	local players_without = {}
	
	Main:Print( L["Your version: "] .. "|cff00ff00" .. self.version .. "|r" )
	
	local count = 0
	
	for player in self:IteratePlayers() do
		local data = self.Status.players[player]
		 
		if data then
			
			if data.compat then
				table.insert( players_with, data.name .. ": |cff00ff00" .. data.version .. "|r" )
			else
				if data.version then
					table.insert( players_without, data.name .. ": |cff900f90" .. data.version .. L[" (Incompatible)"] .. "|r" )
				else
					table.insert( players_without, data.name .. ": |cffff0000" .. L["Not installed."] .. "|r" )
				end
				
			end
		else
			table.insert( players_without, data.name .. ": |cff808080" .. L["Unknown."] .. "|r" )
		end	
		count = count + 1
	end
	
	for _,v in pairs( players_with ) do
		Main:Print( v )
	end
	
	for _,v in pairs( players_without ) do
		Main:Print( v )
	end
end

-------------------------------------------------------------------------------
function Main:SpellSearch( name )
	self.spell_search_text = name
	name = string.lower(name)
	
	local count = 0
	for spellid = 1,200000 do
		local name2 = GetSpellInfo( spellid )
		
		if name2 ~= nil then
			if name == string.lower( name2 ) then
				local name, _, icon = GetSpellInfo( spellid )
				Main:Print( "|T" .. icon .. ":0|t [|cffff0000" .. spellid .. "|r] " .. name )
				count = count + 1
				if count >= 20 then break end
			end
		end
	end
	
	if count == 0 then
		
		Main:Print( L["Unknown spell: "] .. name )
	
	end
	
end

-------------------------------------------------------------------------------
function Main:ShowID( args )
	local name = ""
	for i = 2, #args do
		if name ~= "" then
			name = name .. " "
		end
		name = name .. args[i]
	end
	
	if name == "" then
		Main:Print( L["Usage: "] .. "/delleren id <spell name>" )
		return
	end
	
	local id = self.SpellData:IDFromName( name )
	  
	if id then
		local name, _, icon = GetSpellInfo( id )
		Main:Print( "|T" .. icon .. ":0|t [|cff00ff00" .. id .. "|r] " .. name )
		
	else
		self:SpellSearch( name )
	end
end

-------------------------------------------------------------------------------
-- Slash command for macro binding.
--
function SlashCmdList.DELLEREN( msg )
	local args = {}
	
	for i in string.gmatch( msg, "%S+" ) do
		table.insert( args, i )
	end
	
	if args[1] ~= nil then args[1] = string.lower( args[1] ) end
	
	if args[1] == "call" or args[1] == L["call"] then
	
		Main:CallCommand( args ) 
	
	elseif args[1] == "ignore" or args[1] == L["ignore"] then
	
		Main.Ignore:OpenPanel()
		
	elseif args[1] == "config" or args[1] == L["config"] then
	
		Main.Config:Open()
		
	elseif args[1] == "who" or args[1] == "version" or args[1] == L["who"] or args[1] == L["version"] then
	
		Main:WhoCommand()
		
	elseif args[1] == "id" or args[1] == L["id"] then
	
		Main:ShowID( args )
		
	elseif args[1] == "fuck" or args[1] == L["fuck"] then
		 
		-- this is a police quest reference
		Main:Print( L["My what a filthy mind you have!"] )

	elseif args[1] == "test" then
		 
		Main:Print( "Testing function." ) 

	else
		
		Main:Print( L["Command listing:"] )
		Main:Print( "  /delleren " .. L["config - Open configuration."]  )
		Main:Print( "  /delleren " .. L["call - Call for a cd. (See User's Manual.)"] )
		Main:Print( "  /delleren " .. L["who - List player versions."] )
		Main:Print( "  /delleren " .. L["ignore - Open ignore panel."] )
	
	end
	  
end

-------------------------------------------------------------------------------
function Main:RegisterSlashCommands()
	SLASH_DELLEREN1 = "/delleren"
end
