-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local L = Delleren.Locale

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local SharedMedia = LibStub("LibSharedMedia-3.0")

local VERSION = 2

local TRACKED_SPELL_DEFAULT = {
		{ spell = 6940   };
		{ spell = 33206  };
		{ spell = 102342 };
		{ spell = 114030 };
};

-------------------------------------------------------------------------------
Delleren.Config = {
	 
	spell_search_index = 0;
	spell_search_text  = "";
	general_order = 1000;
  
}

Delleren.Config.db_template = {

	global = {
		version = nil;
	};
	
	char = {
		ignored = {};
	};
	
	profile = {
	
		locked = false;
		
		calling = {	
			whisper    = true;
			localize   = true;
			dellbias   = true;
			--markself   = 0;
			--highlight  = true;
			--mark       = 0;
			--flash      = true;
		}; 
		--[[
		cdbar = {
			size         = 48;
			columns      = 4;
			padding      = 0;
			enable_mouse = true;
			enabled      = false;
			x            = 0;
			y            = -100;
		};]]
		
		tracking = {
			enabled = true;
			list = TRACKED_SPELL_DEFAULT; -- filled in at init
		};
		 
	};
}
 
-------------------------------------------------------------------------------
local OPTIONS_TABLE = {
	type = "group";
	
	args = {
		lockframes = {
			name = L["Unlock Frames"];
			desc = L["Unlocks frames to be moved."];
			type = "execute";
			func = function( info, val ) Delleren:UnlockFrames() end;
			order=1;
		};
		 
		calling = {
			name = L["General"];
			type = "group";
			args = {
				whisperdesc = {
					order = 11;
					name = L["Whisper Option Description"];
					type = "description";
				};
				whisper = {
					order = 12;
					name = L["Enable Whispers"];
					type = "toggle";
					set = function( info, val ) Delleren.Config:SetCallingWhisper( val ) end;
					get = function( info ) return Delleren.Config.db.profile.calling.whisper end;
				};
				
				localizedesc = {
					order = 21;
					name = "\n" .. L["Localize Whisper Description"];
					type = "description";
				};
				localize = {
					order = 22;
					name = L["Localize Whispers"];
					type = "toggle";
					set = function( info, val ) Delleren.Config.db.profile.calling.localize = val end;
					get = function( info ) return Delleren.Config.db.profile.calling.localize end;
				};
				
				dellbiasdesc = {
					order = 31;
					name = "\n" .. L["Prefer Delleren Players Description"];
					type = "description";
				};
				dellbias = {
					order = 32;
					name = L["Prefer Delleren Players"];
					type = "toggle";
					set = function( info, val ) Delleren.Config.db.profile.calling.dellbias = val end;
					get = function( info ) return Delleren.Config.db.profile.calling.dellbias end;
				};
				 
			};
		};
		 
		 
	};
}

Delleren.Config.options = OPTIONS_TABLE

-------------------------------------------------------------------------------
function Delleren.Config:AddGeneral( options )
	for k,v in pairs( options ) do
		if v.order then
			v.order = v.order + self.general_order
		else
			v.order = self.general_order
		end
		
		self.options.args.calling.args[k] = v
	end
	
	self.general_order = self.general_order + 100
end

-------------------------------------------------------------------------------
function Delleren.Config:CreateDB()  

	self.db = LibStub( "AceDB-3.0" ):New( 
					"DellerenAddonSaved", self.db_template, true )
	
	self.db.RegisterCallback( self, "OnProfileChanged", "Apply" )
	self.db.RegisterCallback( self, "OnProfileCopied",  "Apply" )
	self.db.RegisterCallback( self, "OnProfileReset",   "Apply" )
	
	Delleren.db = self.db
	
	-- insert older database patches here: --
	if self.db.global.version < 2 then	
		-- tracking config
		self.db.profile.tracking.list = TRACKED_SPELL_DEFAULT
	end
	-----------------------------------------
	
	self.db.global.version = VERSION
end

-------------------------------------------------------------------------------
function Delleren.Config:InitPanel()
	if self.init then return end
	self.init = true
	 
	Delleren:SendMessage( "DELLEREN_INIT_CONFIG" )
	 
	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable( self.db )
	 
	AceConfig:RegisterOptionsTable( "Delleren", self.options )
end

-------------------------------------------------------------------------------
-- Open the configuration panel.
--
function Delleren.Config:Open()
	self:InitPanel()	
	AceConfigDialog:Open( "Delleren" )
	
	-- hack to fix the scrollbar missing on the first page when you
	-- first open the panel
	self:NotifyChange()
end

function Delleren.Config:NotifyChange()
	LibStub("AceConfigRegistry-3.0"):NotifyChange( "Delleren" )
end
 
-------------------------------------------------------------------------------
-- Apply the configuration settings.
--
function Delleren.Config:Apply( onload )

	local data = self.db.profile
	
	Delleren.Status:UpdateTrackingConfig( onload ) 
	
	Delleren:SendMessage( "DELLEREN_APPLY_CONFIG" )
	
	if not data.locked then 
		Delleren:UnlockFrames()
	end
end
 
-------------------------------------------------------------------------------
function Delleren.Config:SetCallingWhisper( val )
	self.db.profile.calling.whisper = val;
end
 
-------------------------------------------------------------------------------
function Delleren.Config:EnableTracking( enabled )
	if self.db.profile.tracking.enabled == enabled then return end
	self.db.profile.tracking.enabled = enabled
	
	Delleren:SendMessage( "DELLEREN_TRACKING_CHANGED", enabled )
end 

function Delleren.Config:TrackingEnabled()
	return self.db.profile.tracking.enabled
end

function Delleren.Config:GetTrackedSpells()
	return self.db.profile.tracking.list
end

function Delleren.Config:SetTrackedSpells( list )
	self.db.profile.tracking.list = list
	Delleren.Status:UpdateTrackingConfig()
end
