-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local LibMasque = LibStub( "Masque", true )

local Delleren = DellerenAddon
local L = Delleren.Locale
local Me = Delleren:GetModule( "CDView" )

Me.cdbar = {
	frame = nil;
	buttons = {};
	unlocked = false;
	masque_group = nil;
}

Me.db_template.cdbar = {
	size     = 36;
	columns  = 4;
	padding  = 4;
	mouse    = true; 
	position = {};
	masque   = false;
	border_color    = { 0.1875, 0.1875, 0.1875 };
	border_color2   = { 1, 1, 1 };
}

Me.config_section.cdbar = {
	name = L["CD Bar"];
	type = "group";
	inline = true;
	
	args = { 
		
		mouse = {
			name  = L["Enable Mouse"];
			desc  = L["Enable mouse interaction with the CD bar."];
			type  = "toggle";
			set   = function( info, val ) 
				Me.db().cdbar.mouse = val
				if val then
					Me:CDBar_EnableMouse()
				else
					Me:CDBar_DisableMouse()
				end
			end;
			get   = function( info ) return Me.db().cdbar.mouse end;
			order = 4;
		};
		
		spacer1 = {
			name  = "";
			type  = "header";
			order = 5;
		};
		
		size = {
			name  = L["Button Size"];
			desc  = L["Size of button frames."];
			type  = "range";
			min   = 8;
			max   = 128;
			step  = 1;
			set   = function( info, val ) 
				Me.db().cdbar.size = val
				Me:CDBar_UpdateLayout()
			end;
			get   = function( info ) return Me.db().cdbar.size end;
			order = 10;
		};
		
		columns = {
			name  = L["Columns"];
			desc  = L["Number of columns in layout."];
			type  = "range";
			min   = 1;
			max   = 32;
			step  = 1;
			set   = function( info, val ) 
				Me.db().cdbar.columns = val
				Me:CDBar_UpdateLayout()
			end;
			get   = function( info ) return Me.db().cdbar.columns end;
			order = 11;
		};
		
		padding = {
			name  = L["Padding"];
			desc  = L["Padding in between each button."];
			type  = "range";
			min   = -50;
			max   = 50;
			step  = 0.1;
			set   = function( info, val ) 
				Me.db().cdbar.padding = val
				Me:CDBar_UpdateLayout()
			end;
			get   = function( info ) return Me.db().cdbar.padding end;
			order = 12;
		};
		
		border_color = {
			name = L["Border Color"];
			type = "color";
			set = function( info, r, g, b )
				Me.db().cdbar.border_color = {r, g, b}
				Me:CDBar_BorderColorChanged()
			end;
			get = function( info, val )
				return unpack( Me.db().cdbar.border_color )
			end;
		};
		
		border_color2 = {
			name = L["Border Highlight"];
			type = "color";
			set = function( info, r, g, b )
				Me.db().cdbar.border_color2 = {r, g, b} 
			end;
			get = function( info, val )
				return unpack( Me.db().cdbar.border_color2 )
			end;
		};
		
		masque = {
			name = L["Use Masque"];
			desc = L["Skin the buttons using Masque. If you are disabling this, you need to reload your UI. (/reload)"];
			type = "toggle";
			set = function( info, val )
				Me.db().cdbar.masque = val;
				Me:CDBar_UpdateLayout()
			end;
			get = function( info ) return Me.db().cdbar.masque end;
			order = 13;
		};
	};
}

local CDBar = Me.cdbar

function Me:CDBar_BorderColorChanged()
	
	if LibMasque and self.db().cdbar.masque then return end
	
    local c = self.db().cdbar.border_color
	for _,v in pairs( CDBar.buttons ) do
		v.border:SetBackdropBorderColor( c[1], c[2], c[3] ) 
	end
	
end

function Me:CDBar_GetMasque()
	
	if not CDBar.masque_group then
		CDBar.masque_group = LibMasque:Group( "Delleren", "CD Bar" )
	end
	
	return CDBar.masque_group
end

function Me:CDBar_Create()
	if CDBar.frame then return end
	
	CDBar.frame = CreateFrame( "Frame", "DellerenCDBar", UIParent )
	CDBar.frame:SetPoint( "CENTER" )
	CDBar.frame:SetSize( 64, 64 )
	CDBar.frame:SetMovable( true )
	CDBar.frame.OnMoved = function( self )
		Me.db().cdbar.position = Delleren.Utils.GetFramePoints( self )
	end
	self:CDBar_UpdateLayout()
end

function Me:CDBar_UpdateButtons( data )
	self:CDBar_Create()
	
	local relayout = false
	
	for k,v in ipairs( data ) do
		if not CDBar.buttons[k] then
			CDBar.buttons[k] = CreateFrame( "Button", "DellerenCDBarButton" .. k, CDBar.frame, "DellerenCDButtonTemplate" )
			if not self.db().cdbar.mouse then
				CDBar.buttons[k]:EnableMouse( false )
			end
			relayout = true
		end
		
		CDBar.buttons[k]:Update( v )
	end
	
	for i = #data+1, #CDBar.buttons do
		CDBar.buttons[i]:Hide()
	end
	
	if relayout then
		self:CDBar_UpdateLayout()
	end
	
	CDBar.frame:Show()
end

-------------------------------------------------------------------------------
function Me:CDBar_UpdateLayout()
	if not CDBar.frame then return end
	
	local size, columns, padding = self.db().cdbar.size,
								   self.db().cdbar.columns,
								   self.db().cdbar.padding
 
	
	for i = 1, #CDBar.buttons do
		local x = ((i-1) % columns) * (size+padding)
		local y = math.floor((i-1) / columns) * (size+padding)
		
		CDBar.buttons[i]:SetPoint( "TOPLEFT", CDBar.frame, x, -y ) 
		CDBar.buttons[i]:SetSize( size, size )
		
		if LibMasque and self.db().cdbar.masque then
			if not CDBar.buttons[i].masqued then
				CDBar.buttons[i].masqued = true
				self:CDBar_GetMasque():AddButton( CDBar.buttons[i] )
				CDBar.buttons[i].border:SetBackdropBorderColor( 0.0, 0.0, 0.0, 0.0 )
			end
		end
	end
	
	local maxcols = math.min( #Delleren.Config:GetTrackedSpells(), columns )
	maxcols = math.max( maxcols, 1 )
	
	local maxrows = math.floor((#Delleren.Config:GetTrackedSpells()-1)/columns)+1
	
	local framewidth  = math.max( maxcols * (size+padding) - padding, 64 )
	local frameheight = math.max( maxrows * (size+padding) - padding, 32 )
	
	CDBar.frame:SetSize( framewidth, frameheight ) 
	Delleren.Utils.ApplyFramePoints( CDBar.frame, UIParent, self.db().cdbar.position ) 
	
	if self.db().cdbar.mouse then
		self:CDBar_EnableMouse()
	else
		CDBar:DisableMouse()
	end
	
	if self.db().display == "cdbar" and Delleren.Config:TrackingEnabled() then
		CDBar.frame:Show()
	else
		CDBar.frame:Hide()
	end
	
	if LibMasque and self.db().cdbar.masque then
		self:CDBar_GetMasque():ReSkin()
	end
	
end

-------------------------------------------------------------------------------
function Me:CDBar_EnableMouse()
	if self.db().cdbar.mouse then
		for k,v in pairs( CDBar.buttons ) do
			v:EnableMouse( true )
		end
	end
end

-------------------------------------------------------------------------------
function Me:CDBar_DisableMouse()
	for k,v in pairs( CDBar.buttons ) do
		v:EnableMouse( false )
	end
end

-------------------------------------------------------------------------------
function Me:CDBar_Unlock()
	if CDBar.frame then
		if not CDBar.frame.dragger then
			CDBar.frame.dragger = CreateFrame( "Frame", nil, CDBar.frame, "DellerenDraggerTemplate" )
			CDBar.frame.dragger:SetText( "CD Bar" )
		end
		
		CDBar.frame.dragger:Show()
	end
end

-------------------------------------------------------------------------------
function Me:CDBar_Lock()
	if CDBar.frame and CDBar.frame.dragger then
		CDBar.frame.dragger:Hide()
	end
end
