-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local Me = Delleren:GetModule( "CDView" )

-------------------------------------------------------------------------------
function Me.CDButton_OnEnter( self )
	GameTooltip:SetOwner( self, "ANCHOR_NONE" )
    GameTooltip:SetPoint( "TOPLEFT", self, "BOTTOMLEFT" )
	GameTooltip:ClearLines()
	
	local name = GetSpellInfo( self.spell )
	GameTooltip:AddLine( name, 0, 0.7, 1, 1, 1, 1 )
	GameTooltip:AddLine( " " )
	
	local list = Delleren.API:GetSpellStatus( self.spell )
	for _,p in ipairs( list ) do
		
		local left_color, right_color
		local left_text, right_text = p.name, ""
		local _,cls = UnitClass( p.name )
		left_color = RAID_CLASS_COLORS[cls].colorStr
		
		if not p.delleren then
			left_text = left_text .. "*"
		end
		
		if p.dead then
			left_color = "ff404040"
			right_color  = "ff404040"
			if not UnitIsConnected( p.name ) then
				right_text = "OFFLINE"
			else
				right_text = "DEAD"
			end
		elseif p.charges == 0 then
			right_color = "ffc030c0"
			right_text = (tostring(math.ceil( p.cd )) or "???") .. "s"
		elseif not p.inrange then
			right_color = "ffff0000"
			right_text = "OUTRANGED"
		else
			right_color = "ff00ff00"
			
			if p.charges == 1 then
				right_text = "READY"
			else
				right_text = "READY (" .. p.charges .. ")"
			end
		end
		
		GameTooltip:AddDoubleLine( "|c" .. left_color .. left_text, 
		                           "|c" .. right_color .. right_text )
	end
	
	GameTooltip:Show()
end

-------------------------------------------------------------------------------
function Me.CDButton_OnLeave( self )
	GameTooltip:Hide()
end

-------------------------------------------------------------------------------
function Me.CDButton_Update( self, data )

	if self.spell ~= data.spell then
		self.spell = data.spell
		local _,_,icon = GetSpellInfo( data.spell )
		self.icon:SetTexture( icon )
	end
	
	if self.disabled ~= data.disabled then
		if data.disabled then
			self.icon:SetDesaturated( true )
			self:Disable()
		else
			self.icon:SetDesaturated( false )
			self:Enable()
		end
	end
	
	if self.stacks ~= data.stacks or self.time ~= data.time 
	   or self.duration ~= data.duration then
	   
		self.stacks   = data.stacks
		self.time     = data.time
		self.duration = data.duration
		
		if self.time == 0 then
			self.cooldown:SetCooldown( 0, 0, 1, 1 )
			if data.stacks > 1 then
				self.Count:SetText( tostring( data.stacks ))
				self.Count:Show()
			else
				self.Count:Hide()
			end
		else
			self.Count:Hide()
			self.cooldown:SetCooldown( data.time, data.duration, 0, 1 )
		end
	end
	
	if self.outrange ~= data.outrange then
		self.outrange = data.outrange
		if data.outrange then
			self.icon:SetVertexColor( 1,0,0 )
		else
			self.icon:SetVertexColor( 1,1,1 )
		end
	end
	
	self:Show()
end

-------------------------------------------------------------------------------
function Me.CDButton_OnClick( self, button )
	
	if self.stacks == 0 then
		return
	end
	
	local buff = Delleren.SpellData:BuffSpell( self.spell )
	if button == "LeftButton" then
		Delleren.API:Call( {self.spell}, false, buff, false, nil )
	else
		Delleren.API:Call( {self.spell}, false, buff, true, nil )
	end
end
