-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local L = Delleren.Locale

-------------------------------------------------------------------------------
local DisplayModes = {
	[1] = "none";
	[2] = "cdbar";
}

local DisplayModeNames = {
	[1] = "Disabled";
	[2] = "Action Bar";
}

local DisplayModesMap = {
	none = 1;
	cdbar = 2;
}

-------------------------------------------------------------------------------
local Me = {

	db_template = {
		display = "cdbar"; 
	};
}

-------------------------------------------------------------------------------
Me.config_section = {
		
	desc = {
		name  = L["Options for displaying CDs that you are tracking. Tracking must be enabled for this feature to work."];
		type  = "description";
		order = 1;
	};
	  
	mode = {
		name = L["Display Mode"];
		type = "select";
		order = 3;
		values = DisplayModeNames; 
		set = function( info, val )
			Me:SetDisplayMode( DisplayModes[val] )
		end;
		get = function( info ) return DisplayModesMap[Me:db().display] end
	}; 
}

-------------------------------------------------------------------------------
function Me:OnStart()
	if self.db().display == "cdbar" then
		self:CDBar_Create()
	end
end

-------------------------------------------------------------------------------
function Me:SetDisplayMode( mode, force )
	if self.db().display == mode and not force then return end
	self.db().display = mode
	
	self.config_section.cdbar.hidden = true
	
	if mode == "cdbar" then
		self.config_section.cdbar.hidden = false
	end
	 
	self:CDBar_UpdateLayout()
	Delleren.Config:NotifyChange()
end
 
-------------------------------------------------------------------------------
DellerenAddon:RegisterModule( "CDView", Me, {
	db = Me.db_template;
	config_section = Me.config_section;
})

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CD_DATA( data )
	if self.db().display == "cdbar" then
		
		self:CDBar_UpdateButtons( data )
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_UNLOCKED()
	
	self:CDBar_Unlock()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_LOCKED()
	self:CDBar_Lock()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_TRACKING_CHANGED( enabled )
	if not enabled then
		
		self:CDBar_UpdateLayout()
	end
end	

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_APPLY_CONFIG()
	Me:SetDisplayMode( Me.db().display, true )
end
 