-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

-- unitframe flasher

local Delleren = DellerenAddon
local L = Delleren.Locale

local Me = {
	
	active    = false;
	player    = nil;
	frames    = {};		-- list of frames we can use
	fcount    = 0;		-- number of frames that are being used
	flashtime = 0;
};

-------------------------------------------------------------------------------
-- Iterate over unit buttons that are linked to a certain player.
--
-- @param name Name of player.
--
local function IterateUnitButtons( name )
	local raidid = UnitInRaid( name )
	--if not raidid then return function() end end
	if not raidid then
		raidid = ""
	else
		raidid = "raid" .. raidid
	end
	
	local frame = nil
	
	return function()
	
		while true do
			
			frame = EnumerateFrames( frame )
			if not frame then return end
			
			if frame:HasScript( "OnClick" ) 
			   and frame:GetScript( "OnClick" ) == SecureUnitButton_OnClick then
			   
				local unit = frame:GetAttribute( "unit" )
				if unit then
					if (unit == raidid) 
					   or (unit:find("party") and UnitGUID( unit ) == UnitGUID( name )) then 
					   
						return frame 
					end
					
				end
			    --thanks Semler!
			    
			end
		end
	end
end
 
-------------------------------------------------------------------------------
function Me:SetupFrame( index, parent, spell, isitem )
	
	local frame = self.frames[index]
	if not frame then
		frame = CreateFrame( "Frame", nil, UIParent, "DellerenFlasherTemplate" ) 
		self.frames[index] = frame
	end
	 
	frame:ClearAllPoints()
	frame:SetParent( parent )
	frame:SetAllPoints()
	frame:SetIcon( spell, isitem )
	frame:Show() 
end

-------------------------------------------------------------------------------
-- Setup the module to flash a given player.
--
-- @param name Name of player.
--
function Me:Start( name, spell, isitem )
	
	local count = 0
	for frame in IterateUnitButtons( name ) do
	
		count = count + 1
		self:SetupFrame( count, frame, spell, isitem )
	end
	self.fcount = count
	
	for i = self.fcount+1,#self.frames do
		self.frames[i]:Hide()
	end
	
	self.flashtime = GetTime()
	
	self.active = true
end

-------------------------------------------------------------------------------
-- Emit a flash.
--
function Me:Flash()
	self.flashtime = GetTime()
end

-------------------------------------------------------------------------------
-- Frame update function.
--
function Me:Update()
	if not self.active then return end
	 
end

-------------------------------------------------------------------------------
-- Stop flashing a unit.
--
function Me:Stop()
	if not self.active then return end
	
	for _,frame in pairs(self.frames) do
		frame:Hide()
	end
	self.active = false
end

-------------------------------------------------------------------------------
function Me.Frame_SetIcon( self, id, isitem )

	self.poker.icon:SetTexture( Delleren.Utils.GetIcon( id, isitem ) )
	
	-- zoom
	self.poker.icon:SetTexCoord( 0.05, 0.95, 0.05, 0.95 )
end

-------------------------------------------------------------------------------
function Me.Frame_OnShow( self )
	local sw,sh = UIParent:GetWidth(), UIParent:GetHeight()
    local mx,my = self:GetCenter() 
	
	if not mx then
		mx = 0
		my = 0
	end
	
	-- where to mount the arrow
	
	self.poker.arrow:ClearAllPoints()  
	self.poker:ClearAllPoints()
	
	if mx < sw * 0.33 then
		-- topleft, left, bottomleft
		-- mount to left
		self.direction = 0
		self.poker.arrow:SetPoint( "RIGHT", self.poker, "LEFT" ) 
		self.poker.arrow:SetTexCoord( 1, 1, 1, 0, 0, 1, 0, 0 )
		
	elseif mx > sw * 0.66 then
		-- topright, right, bottom right
		-- mount to right
		self.direction = 2
		self.poker.arrow:SetPoint( "LEFT", self.poker, "RIGHT" )
		self.poker.arrow:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 )
		
	elseif my > sh * 0.66 then
		-- top
		-- mount to top
		self.direction = 1
		self.poker.arrow:SetPoint( "BOTTOM", self.poker, "TOP" )
		self.poker.arrow:SetTexCoord( 1, 0, 0, 0, 1, 1, 0, 1 )
		
	else
		-- middle, bottom
		-- mount to bottom
		self.direction = 3
		self.poker.arrow:SetPoint( "TOP", self.poker, "BOTTOM" ) 
		self.poker.arrow:SetTexCoord( 0, 1, 1, 1, 0, 0, 1, 0 )
		
		
	end
	
	self.showtime = GetTime()
	Me.Frame_UpdatePokerPosition( self )
	
	self:SetFrameLevel( self:GetParent():GetFrameLevel() + 10 )
end

-------------------------------------------------------------------------------
function Me.Frame_UpdatePokerPosition( self )
	local t = GetTime() - Me.flashtime
	
	local offset = -0.32
	local delta = (1 + math.sin( math.pi*(1.5+offset) + t * math.pi*2 )) / 2
	
	local slide = 25
	
	if self.direction == 0 then
		-- pointing left
		self.poker:SetPoint( "LEFT", self, "RIGHT", 3 + delta * slide, 0 )
		
	elseif self.direction == 1 then
		-- pointing up
		self.poker:SetPoint( "TOP", self, "BOTTOM", 0, -3 - delta * slide )
		
	elseif self.direction == 2 then
		-- pointing right
		self.poker:SetPoint( "RIGHT", self, "LEFT", -3 - delta * slide, 0 )
	elseif self.direction == 3 then
		-- pointing down
		self.poker:SetPoint( "BOTTOM", self, "TOP", 0, 3 + delta * slide )
	end	
end

-------------------------------------------------------------------------------
function Me.Frame_OnUpdate( self )
	local t = GetTime() - self.showtime
	
	Me.Frame_UpdatePokerPosition( self )
	
	local flash = GetTime() - Me.flashtime 
	flash = flash % 1.0
	if flash >= 0.4 then flash = flash - 0.4 end
	flash = flash * 2.0
	flash = 1.0 - flash
	flash = math.min( flash, 1.0 )
	flash = math.max( flash, 0.0 )  
	flash = math.pow( flash, 2.5 ) 
	flash = math.min( flash, 1.0 )
	flash = math.max( flash, 0.0 )
	self.tex:SetColorTexture( flash, flash, flash, 1 )
	
end

-------------------------------------------------------------------------------
Delleren:RegisterModule( "Flasher", Me, {
	name = "Flasher";
	db = {
		enabled = true;
	};
	config_general = {
		flashdesc = {
			order = 1;
			name = "\n" .. L["Flash Unit Desc"];
			type = "description";
		};
		flash = {
			order = 2;
			name = L["Flash Unit Frames"];
			type = "toggle";
			set = function( info, val ) Me.db().enabled = val end;
			get = function( info ) return Me.db().enabled end;
		};
	};
})

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_START( data )

	if Me.db().enabled then
		self:Start( data.name, data.id, data.item )
		self:Flash()
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_PULSE()
	if Me.db().enabled then
		self:Flash()
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_END()
	self:Stop()
end
