-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local L = Delleren.Locale
local SharedMedia = LibStub("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
local Me = {
	frame = nil;
	mode = nil;
}

-------------------------------------------------------------------------------
function Me:OnStart()
	
	self.frame = CreateFrame( "Button", "DellerenIndicator", UIParent, "DellerenIndicatorTemplate" )
end

-------------------------------------------------------------------------------
function Me:LoadConfig()
	local size = math.max( self.db().size, 16 )
	size = math.min( size, 256 )
	self.frame:SetSize( size, size )
	
	Delleren.Utils.ApplyFramePoints( self.frame, UIParent, self.db().position )
	
	local size = self.db().fontsize
	size = math.max( size, 4 )
	size = math.min( size, 32 )
	
	self.frame.text:SetFont( 
		SharedMedia:Fetch( "font", self.db().font ),
		size,
		"OUTLINE" )
end
-------------------------------------------------------------------------------
local function ColorLerp( r1, g1, b1, r2, g2, b2, a )
	a = math.min( a, 1 )
	a = math.max( a, 0 )
	return r1 + (r2-r1) * a, 
	       g1 + (g2-g1) * a, 
		   b1 + (b2-b1) * a
end

-------------------------------------------------------------------------------
function Me.Frame_OnShow( self )
	
end

-------------------------------------------------------------------------------
function Me.Frame_SetAnimation( self, state )
	self.ani.state    = state
	self.ani.time     = GetTime()
	self.ani.finished = false
end

function Me.Frame_SetIcon( self, id, item )
	
	self.icon:SetTexture( Delleren.Utils.GetIcon( id, isitem ) )
	
	-- zoom
	self.icon:SetTexCoord( 0.06, 0.94, 0.06, 0.94 )
end

-------------------------------------------------------------------------------
function Me.Frame_OnUpdate( self )
	local t = GetTime() - self.ani.time
	
	-- animation magic
	if self.ani.state == "CALL" then
	
		local r,g,b = ColorLerp( 1,1,1, 1,0.7,0.2, t / 0.25 )
		self.icon:SetVertexColor( r, g, b, 1 )
		self:SetAlpha( 1 )
		self.text:SetTextColor( 1, 1, 1, 1 )
		self.icon:SetDesaturated( true ) 
		if t >= 1.0 then self.ani.finished = true end
		
	elseif self.ani.state == "MANCALL" then
		local r,g,b = 0.9, 0.2, 0.05
		
		r = r + math.sin( GetTime() * 6.28 * 4.5 ) * 0.7
		r,g,b = ColorLerp( 1,1,1,r,g,b, t / 0.2 )
		self.icon:SetVertexColor( r, g, b, 1 )
		self:SetAlpha( 1 )
		self.text:SetTextColor( 1,1,1,1 )
		self.icon:SetDesaturated( true ) 
		if t >= 1.0 then self.ani.finished = true end
		
	elseif self.ani.state == "SUCCESS" then
		
		local a = 1.0 - math.min( t / 0.5, 1 )
		self.icon:SetVertexColor( 0.3, 1, 0.3, 1 )
		self:SetAlpha( a )
		self.text:SetTextColor  ( 0.3, 1, 0.3, 1 )
		self.icon:SetDesaturated( true ) 
		if t >= 0.5 then 
			self.ani.finished = true 
			self:Hide()
		end
		
	elseif self.ani.state == "FAILURE" then
	
		local a = 1.0 - math.min( t / 0.5, 1 )
		self.icon:SetVertexColor( 1, 0.1, 0.2, 1)
		self:SetAlpha( a )
		self.text:SetTextColor  ( 1, 0.1, 0.2, 1 )
		self.icon:SetDesaturated( true ) 
		if t >= 0.5 then 
			self.ani.finished = true 
			self:Hide()
		end
		
	elseif self.ani.state == "POLLING" then
	
		local r,g,b = 0.25, 0.25, 0.6
		
		b = b + math.sin( GetTime() * 6.28 * 3 ) * 0.4
		
		r,g,b = ColorLerp( 1,1,1,r,g,b, t / 0.2 )
		self.icon:SetVertexColor( r, g, b, 1 )
		self:SetAlpha( 1 )
		self.text:SetTextColor( 1,1,1,1 )
		self.icon:SetDesaturated( true ) 
		if t >= 1.0 then self.ani.finished = true end
		
	elseif self.ani.state == "HELP" then
		local r,g,b = ColorLerp( 1,1,1, 0.5,0,0.5, t/0.25 )
		self.icon:SetVertexColor( r, g, b, 1 )
		self.text:SetTextColor( 1,1,1,1 )
		self:SetAlpha( 1 )
		self.icon:SetDesaturated( false )
		if t >= 1.0 then self.ani.finished = true end
	else
		self.text:SetTextColor( 1,1,1,1 )
	end
end

-------------------------------------------------------------------------------
function Me.Frame_OnMoved( self )
	Me.db().position = Delleren.Utils.GetFramePoints( self )
end

-------------------------------------------------------------------------------
Delleren:RegisterModule( "Indicator", Me, {
	name = "Indicator";
	
	db = {
		size = 60;
		fontsize = 14;
		font = "Arial Narrow"; 
		position = {};
		enabled = true;
	};
	
	config_section = {
		
		framesize = {
			name  = L["Frame Size"];
			desc  = L["Size of the indicator frame."];
			type  = "range";
			min   = 16;
			max   = 256;
			step  = 1;
			set   = function( info, val ) 
				Me.db().size = val
				Me:LoadConfig()
			end;
			get   = function( info ) return Me.db().size end;
			order = 1;
		};
		
		fontsize = {
			name  = L["Font Size"];
			desc  = L["Size of the indicator font."];
			type  = "range";
			min   = 4;
			max   = 24;
			step  = 1;
			set   = function( info, val ) 
				Me.db().fontsize = val;
				Me:LoadConfig()
			end;
			get   = function( info ) return Me.db().fontsize end;
			order = 2;
		};
		
		fontface = {
			name  = L["Font"];
			desc  = L["Indicator font."];
			type  = "select"; 
			set   = function( info, val ) 
				
				Me.db().font = Delleren:GetModuleConfigOption( "Indicator", "fontface" ).values[val];
				Me:LoadConfig()
			end;
			get   = function( info ) 
				return Delleren.Utils.FindValueKey( 
					Delleren:GetModuleConfigOption( "Indicator", "fontface" ).values, 
					Me.db().font ) 
			end;
			order = 3;
		};
		
		resetpos = {
			name  = L["Reset Position"];
			desc  = L["Reset the indicator's position."];
			type  = "execute";
			func  = function() 
				Me.db().position = {}
				Me:LoadConfig()
			end;
			order = 4;
		};
	};
})
 
-------------------------------------------------------------------------------
function Me:Event_DELLEREN_INIT_CONFIG()
	Delleren:GetModuleConfigOption( "Indicator", "fontface" ).values = SharedMedia:List( "font"  )
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_APPLY_CONFIG()
	self:LoadConfig()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_START( data ) 
	if self.mode == "help" then return end -- ignore when in help mode
	self.mode = "query"
	
	self.frame.text:SetText( "" )
	self.frame:SetAnimation( "POLLING" )
	self.frame:SetIcon( data.list[1], data.item )
	self.frame:Show()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_CHECK( data ) 
	if self.mode ~= "query" then return end
	self.frame:SetIcon( data.list[1], data.item )
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_REQ_START( data ) 
	if self.mode ~= "query" then return end
	
	local unit_name = Delleren.Utils.UnitNameColored( data.name )
	local request_text = Delleren.Utils.GetSpellName( data.id, data.item )
	
	self.frame.text:SetText( unit_name .. "\n" .. request_text  )
	self.frame:SetIcon( Delleren.Utils.GetIcon( data.id, data.item ) )
	
	if data.compat then
		self.frame:SetAnimation( "CALL" )
	else
		self.frame:SetAnimation( "MANCALL" )
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_END( data ) 
	if self.mode ~= "query" then return end
	
	if data.success then
		self.frame:SetAnimation( "SUCCESS" )
	else
		self.frame:SetAnimation( "FAILURE" )
	end 
	self.mode = nil
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_START( data ) 
	self.mode = "help"
	
	local unit_name = Delleren.Utils.UnitNameColored( data.name )
	local request_text = Delleren.Utils.GetSpellName( data.id, data.item )
	self.frame.text:SetText( request_text .. "\n" .. unit_name )
	self.frame:SetIcon( data.id, data.item )
	
	self.frame:SetAnimation( "HELP" )
	self.frame:Show()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_PULSE() 
	if self.mode == "help" then
		self.frame:SetAnimation( "HELP" )
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_END( data ) 
	if self.mode == "help" then
		if data.failed then
			self.frame:SetAnimation( "FAILURE" )
		else
			self.frame:SetAnimation( "SUCCESS" )
		end
	end
	self.mode = nil
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_UNLOCKED()
	if not self.frame.dragger then
		self.frame.dragger = CreateFrame( "Frame", nil, self.frame, "DellerenDraggerTemplate" )
		self.frame.dragger:SetText( "" )
	end
	
	self.frame:Show()
	if not self.mode then
		self.frame:SetAnimation( "POLLING" )
		self.frame.text:SetText( "Delleren\nIndicator" )
	end
	self.frame.dragger:Show()
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_LOCKED()
	if not self.mode then
		self.frame:Hide()
	end
	self.frame.dragger:Hide()
end
