-- raid markers and raid announcements

local Delleren = DellerenAddon
local L = Delleren.Locale

local Me = {
	oldmark = nil;
}

-------------------------------------------------------------------------------
-- Mark self according to config.
--
function Me:SetMarker()

	if Delleren.Utils.NoRaidAssist() then return end
	
	local mark = Me.db().mark
	if mark == 0 then return end 
	
	self.oldmark = GetRaidTargetIndex( "player" )
	SetRaidTarget( "player", 0 )
	SetRaidTarget( "player", mark )
end

-------------------------------------------------------------------------------
-- Clear self mark.
--
function Me:ClearMarker()
	if Delleren.Utils.NoRaidAssist() then return end
	
	local mark = Me.db().mark
	if mark == 0 then return end 
	
	if GetRaidTargetIndex( "player" ) == mark then
		SetRaidTarget( "player", 0 )
		if self.oldmark then
			SetRaidTarget( "player", self.oldmark )
		end
	end
end

-------------------------------------------------------------------------------
-- Announce the call to the raid.
--
-- @param data Data from call event.
--
function Me:AnnounceRaid( data )
	
	if not IsInRaid() then return end
	
	local name = Delleren.Utils.GetSpellName( data.id, data.item )
	
	local msg
	if data.buff then
		msg = L( "{1}, give {2} {3}.", data.name, UnitName( "player" ), name )
	else
		msg = L( "{1}, use {2}.", data.name, name )
	end

	if not Delleren.Utils.NoRaidAssist() then 	
		SendChatMessage( msg, "RAID_WARNING" )
	elseif Me.db().raidwarn_fallback then
		SendChatMessage( msg, "RAID" )
	else
		-- all that work for nothing :)
	end
end
 
-------------------------------------------------------------------------------
Delleren:RegisterModule( "RaidMarker", Me, {
	name = "Raid Marker";
	db = {
		mark = 0;
		raidwarn_d = false;
		raidwarn_n = false;
		raidwarn_fallback = true;
	};
	config_general = {
		
		raidwarndesc = {
			order = 1;
			name = "\n" .. L["Raid Warning Desc"];
			type = "description";
		};
		raidwarn_nodell = {
			order = 2;
			name = L["Announce when calling from players without Delleren"];
			type = "toggle";
			width = "full";
			set = function( info, val ) Me.db().raidwarn_n = val end;
			get = function( info ) return Me.db().raidwarn_n end;
		};
		raidwarn_dell = {
			order = 3;
			name = L["Announce when calling from players with Delleren"];
			type = "toggle";
			width = "full";
			set = function( info, val ) Me.db().raidwarn_d = val end;
			get = function( info ) return Me.db().raidwarn_d end;
		};
		raidwarn_fallback = {
			order = 4;
			name = L["Fallback to raid-chat"];
			type = "toggle";
			width = "full";
			set = function( info, val ) Me.db().raidwarn_fallback = val end;
			get = function( info ) return Me.db().raidwarn_fallback end;
		};
		
		markdesc = {
			order = 5;
			name = "\n" .. L["Raid Marker Desc"];
			type = "description";
		};
		
		mark = {
			order = 6;
			name = L["Raid Marker"];
			type = "select";
			values = { 
				[0] = L["Don't Mark"];
				[1] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_1:0|t " .. RAID_TARGET_1;
				[2] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_2:0|t " .. RAID_TARGET_2;
				[3] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_3:0|t " .. RAID_TARGET_3;
				[4] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_4:0|t " .. RAID_TARGET_4; 
				[5] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_5:0|t " .. RAID_TARGET_5;
				[6] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_6:0|t " .. RAID_TARGET_6;
				[7] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:0|t " .. RAID_TARGET_7;
				[8] = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:0|t " .. RAID_TARGET_8;
			};
					   
			set = function( info, val ) Me.db().mark = val end;
			get = function( info ) return Me.db().mark end;
		};
	};
})

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_REQ_START( data )
	self:SetMarker()
	
	if (data.compat and Me.db().raidwarn_d) or Me.db().raidwarn_n then
		self:AnnounceRaid( data )
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_REQ_END()
	self:ClearMarker()
end
