-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com) 
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local L = Delleren.Locale

-------------------------------------------------------------------------------
local Me = {} -- Stateless!

-------------------------------------------------------------------------------
local SharedMedia = LibStub("LibSharedMedia-3.0")

SharedMedia:Register( "sound", "Delleren-Call",    "Interface\\Addons\\Delleren\\sounds\\ask.ogg"     )
SharedMedia:Register( "sound", "Delleren-ManCall", "Interface\\Addons\\Delleren\\sounds\\mancall.ogg" )
SharedMedia:Register( "sound", "Delleren-Help",    "Interface\\Addons\\Delleren\\sounds\\help.ogg"    )
SharedMedia:Register( "sound", "Delleren-Fail",    "Interface\\Addons\\Delleren\\sounds\\fail.ogg"    )
SharedMedia:Register( "sound", "Delleren-Good",    "Interface\\Addons\\Delleren\\sounds\\good.ogg"    )

-------------------------------------------------------------------------------
local SOUND_CHANNELS = {
	["Master"]   = L["Master"];
	["SFX"]      = L["Sound Effects"];
	["Ambience"] = L["Ambience"];
	["Music"]    = L["Music"];
}

local CONFIG_SECTION = {
	channel = {
		name   = L["Sound Channel:"];
		desc   = L["Channel to play sounds on."];
		type   = "select";
		values = SOUND_CHANNELS;
		set    = function( info, val ) Delleren.db.profile.Sounds.channel = val end;
		get    = function( info ) return Delleren.db.profile.Sounds.channel end;
		order  = 1;
	}; 
}

-------------------------------------------------------------------------------
do 
	local order = 10
	local function InsertSoundOption( key, name, desc )
	 
		CONFIG_SECTION[key .. "_BREAK"] = {
			type = "header";
			name = "";
			order = order;
		}
		
		CONFIG_SECTION[key] = {
			name = name;
			desc = desc;
			type = "select";
			values = "SOUND_LISTING"; -- replaced later
			set  = function( info, val ) 
				val = SharedMedia:List( "sound" )[val]
				Delleren.db.profile.Sounds.sounds[key].name = val
				Me:PlaySound( key )
			end;
			get  = function( info ) 
				return Delleren.Utils.FindValueKey( 
						SharedMedia:List( "sound" ), 
						Delleren.db.profile.Sounds.sounds[key].name ) 
			end;
			order = order+1;
		}
		 
		CONFIG_SECTION[key .. "_ENABLE"] = {
			name = "Enable";
			desc = "Enable this sound.";
			type = "toggle";
			set  = function( info, val ) 
				Delleren.db.profile.Sounds.sounds[key].enabled = val
				if val then
					Me:PlaySound( key )
				end
			end;
			get  = function( info ) 
				return Delleren.db.profile.Sounds.sounds[key].enabled 
			end;
			order = order+2;
		}
		
		order = order + 10
		
	end
	
	InsertSoundOption( "CALL", L["Call:"], L["Sound to play when making a call."] )
	InsertSoundOption( "MANCALL", L["Non-Delleren Call:"], L["Sound to play when making a call to a user without Delleren."] )
	InsertSoundOption( "HELP", L["Help:"], L["Sound to play when being asked for help."] )
	InsertSoundOption( "FAIL", L["Fail:"], L["Sound to play when something goes wrong."] )

end


-------------------------------------------------------------------------------
function Me:PlaySound( sound )
	local cfg = Delleren.db.profile.Sounds
	local info = cfg.sounds[sound]
	
	if info.enabled then
		PlaySoundFile( SharedMedia:Fetch( "sound", info.name ), cfg.channel )
	end	
end

Delleren:RegisterModule( "Sounds", Me, {
	name = "Sounds";
	
	db = {
		channel = "Master";

		sounds = {
			
			CALL = {
				name    = "Delleren-Call";
				file    = nil; 
				enabled = true;
			};
			MANCALL = {
				name    = "Delleren-ManCall";
				file    = nil;
				enabled = true;
			};
			HELP = {
				name    = "Delleren-Help";
				file    = nil; 
				enabled = true;
			};
			FAIL = {
				name    = "Delleren-Fail";
				file    = nil; 
				enabled = true
			};
		};
	};
	
	config_section = CONFIG_SECTION;
})

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_INIT_CONFIG()

	local cs = Delleren:GetModuleConfigSection( "Sounds" )
	 
	for k,v in pairs( cs ) do
		if v.values == "SOUND_LISTING" then
			v.values = SharedMedia:List( "sound" )
		end
	end
	
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_END( data )
	if data.failed then
		self:PlaySound( "FAIL" )
	else
		
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_HELP_PULSE()
	self:PlaySound( "HELP" )
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_END( data )
	if data.success then
		--self:PlaySound( "GOOD" )
	else
		self:PlaySound( "FAIL" )
	end
end

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_CALL_REQ_START( data )
	if data.compat then
		self:PlaySound( "CALL" )
	else
		self:PlaySound( "MANCALL" )
	end
	
end

