-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

local Delleren = DellerenAddon
local L = Delleren.Locale

local Me = {
	editor_text = "";
}

local CONFIG_SECTION = {
	desc = {
		order = 1;
		name  = L["Tracked Spells Description"];
		type  = "description";
	};
	
	enabled = {
		order = 2;
		name = L["Enabled"];
		type = "toggle";
		set = function( info, val )
			if val then 
				Delleren.Config:EnableTracking(true)
			else
				Delleren.Config:EnableTracking(false)
			end
		end;
		get = function( info )
			return Delleren.Config:TrackingEnabled()
		end;
	};
	
	split1 = {
		order = 4;
		name  = L["Tracked Spells"];
		type  = "header";
	};
	
	split2 = {
		order = 1000;
		name  = L["Edit"];
		type  = "header";
	};
	
	editorhelp = {
		order = 1001;
		name  = L["Editor Help"];
		type  = "description";
	};
	
	editor = {
		order = 1002;
		name  = "";
		desc  = "";
		type  = "input";
		multiline = 10;
		width = "full";
		get   = function( info ) return Me.editor_text end;
		set   = function( info, val )
			Me:EditorChanged( val )
		
		end;
	};
}

-------------------------------------------------------------------------------
local function trim1(s)
	return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-------------------------------------------------------------------------------
function Me:EditorChanged( val )
	self.editor_text = val
	
	local list = {}
	
	for word in string.gmatch( val, "[^\n]+" ) do
		word = trim1(word)
		local spellid = tonumber(word)
		if not spellid then
			spellid = Delleren.SpellData:IDFromName( word )
			if not spellid then
				Delleren:Print( L["Unknown spell:"] .. ' "' .. word .. '"' )
			end
		end
		
		if spellid ~= nil then
			
			if not Delleren.SpellData:KnownSpell( spellid ) then
				local name = GetSpellInfo( spellid )
				
				if name then
					Delleren:Print( L["Unknown spell:"] .. " " .. name )
				else
					Delleren:Print( L["Invalid spell ID:"] .. " " .. spellid )
				end
				
				spellid = nil
			else
			
				--if spellid ~= nil and GetSpellInfo( spellid ) then
			
				local duplicate = false
				
				for k,v in pairs( list ) do
					if v.spell == spellid then
						duplicate = true
						break
					end
				end
				
				if not duplicate then
					table.insert( list, {spell=spellid} )
				end
			end
		end
	end
	
	Delleren.Config:SetTrackedSpells( list ) 
	
	self:RebuildTrackedSpellOptions()  
end

-------------------------------------------------------------------------------
function Me:ResetTrackedSpellOptions()
	for k,v in pairs( CONFIG_SECTION ) do
		if string.find( k, "trackedspell" ) then
			CONFIG_SECTION[k] = nil
		end
	end
	self.config_tracked_spell_index = 1
end

-------------------------------------------------------------------------------
function Me:InsertTrackedSpellOption( spellid )
	local prefix = "trackedspell" .. self.config_tracked_spell_index
	local order = 10 + self.config_tracked_spell_index * 10
	
	local name,_,icon = GetSpellInfo( spellid )
	
	CONFIG_SECTION[prefix .. "desc"] = {
		order = order;
		type = "description";
		name = name;
		image = icon;
		imageWidth = 20;
		imageHeight = 20;
		fontSize = "large";
	} 
	
	self.config_tracked_spell_index = self.config_tracked_spell_index + 1
end

-------------------------------------------------------------------------------
function Me:RebuildTrackedSpellOptions()
	self:ResetTrackedSpellOptions()
	
	for k,v in ipairs( Delleren.Config:GetTrackedSpells() ) do
		self:InsertTrackedSpellOption( v.spell )
	end
end

-------------------------------------------------------------------------------
function Me:ResetTrackingEditorText()
	self.editor_text = ""
	for k,v in ipairs( Delleren.Config:GetTrackedSpells() ) do
		self.editor_text = 
			self.editor_text .. string.lower(GetSpellInfo(v.spell)) .. "\n"
	end
end

-------------------------------------------------------------------------------
Delleren:RegisterModule( "TrackingConfig", Me, {
	name = "Tracking"; 
	config_section = CONFIG_SECTION;
});

-------------------------------------------------------------------------------
function Me:Event_DELLEREN_INIT_CONFIG()
	Me:ResetTrackingEditorText()
	Me:RebuildTrackedSpellOptions()
end
