-------------------------------------------------------------------------------
-- DELLEREN
-- (C) 2016 Mukunda Johnson (mukunda@mukunda.com)
-- See LICENSE-DELLEREN.TXT
-------------------------------------------------------------------------------

-- special thanks to Barkskin-Mannoroth for helping me test some of them spells.

local Delleren = DellerenAddon

--spell definition format:
--
-- [spellid] = { 
--    cd      = cooldown in seconds, default = GetSpellBaseCooldown
--    talent  = Talent required to activate spell, formatted as row * 10 + column 
--    charges = max number of spell charges, default (nil) is 1
--    mods    = list of functions that modify the data according to talent data
--    buff    = spellid of buff we expect to be cast on us, default = same as spellid, 0 = non-buff ability
-- }

Delleren.SpellData = {}

-------------------------------------------------------------------------------
local function TalentSelected( build, index )

	local row = math.floor( index / 10 )
	index = index - row * 10
	return tonumber(string.sub( build.talents, row, row )) == index
	
--	local _,_,_,sel,avail = GetTalentInfo( row, index, nil, true, name )
--	return sel and avail
end
  
local CAT_DEF      = 1 -- Defensive CD
local CAT_RAIDDEF  = 2 -- Raid defensive CD
local CAT_UTIL     = 3 -- Utility CD 
local CAT_SAVE     = 4 -- Immunities
local CAT_RAIDHEAL = 5 -- Raid Heal CD
local CAT_AOESTUN  = 6 -- area stun

-------------------------------------------------------------------------------
local SpellData = {

-------------------------------------------------------------------------------
	["DEATHKNIGHT"] = {
		all = {
			
		};
	};
	
-------------------------------------------------------------------------------
	["DEMONHUNTER"] = {
		all = {
			[196718] = { cat=CAT_RAIDDEF; buff=0; };           -- Darkness
		};
	};
	
-------------------------------------------------------------------------------
	["DRUID"] = {
	
		-- feral
		[103] = {
			[106898] = { cat=CAT_UTIL; buff=0; };           -- Stampeding Roar
		};
		
		-- guardian
		[104] = {
			[106898] = { cat=CAT_UTIL; buff=0; };           -- Stampeding Roar
		};
	
		-- resto
		[105] = {
			[740]    = { cat=CAT_RAIDHEAL; buff=0; };       -- Tranquility
			[102342] = { cat=CAT_DEF; };                    -- Ironbark
		};
	};
	
-------------------------------------------------------------------------------
	["HUNTER"] = {
	};
	
-------------------------------------------------------------------------------
	["MAGE"] = {
		all = {
			[113724] = { cat=CAT_UTIL; talent = 52; buff=0; }; -- Ring of Frost
		};
	};
	
-------------------------------------------------------------------------------
	["MONK"] = {
		all = {
			[116841] = { cat=CAT_UTIL; talent = 22; };            -- Tiger's Lust
			[116844] = { cat=CAT_UTIL; talent = 41; };            -- Ring of Peace 
			[119381] = { cat=CAT_AOESTUN; talent = 43; buff=0; }; -- Leg Sweep
		};
		
		-- mistweaver
		[270] = {
			[115310] = { cat=CAT_RAIDHEAL; buff=0 };            -- Revival
			[116849] = { cat=CAT_DEF; };                        -- Life Cocoon
		};
	};
	
-------------------------------------------------------------------------------
	["PALADIN"] = {
		all = {
			[1044]   = { cat=CAT_UTIL; };     -- Blessing of Freedom
			[1022]   = { cat=CAT_SAVE; };     -- Blessing of Protection
			[633]    = { cat=CAT_SAVE; buff=0; };                    -- Lay on Hands 
		};
		
		-- protection
		[66] = { 
			[204013] = { cat=CAT_UTIL; talent = 42};   -- Blessing of Salvation
			[6940]   = { cat=CAT_DEF; };               -- Blessing of Sacrifice
		};
		
		-- holy
		[65] = {
			[6940]   = { cat=CAT_DEF; };     -- Blessing of Sacrifice
		};
	};
	
-------------------------------------------------------------------------------
	["PRIEST"] = {
	
		-- disc
		[256] = {
			[62618] = { cat=CAT_RAIDDEF; buff=0; };         -- Power Word: Barrier
			[33206] = { cat=CAT_DEF; };                     -- Pain Suppression
		};
		
		-- holy
		[257] = {
			[64843] = { cat=CAT_RAIDHEAL; buff=0; };        -- Divine Hymn
			[47788] = { cat=CAT_SAVE; };                    -- Guardian Spirit
		};
	};
	
-------------------------------------------------------------------------------
	["ROGUE"] = {
		all = { 
			[1725]  = { cat=CAT_UTIL; buff=0; };            -- Distract
		};
	};
	
-------------------------------------------------------------------------------
	["SHAMAN"] = {
	
		-- resto
		[264] = {
			[98008] = { cat=CAT_RAIDDEF; buff=0; };         -- Spirit Link Totem
		};
	};
	
-------------------------------------------------------------------------------
	["WARLOCK"] = {
		
	};
	
-------------------------------------------------------------------------------
	["WARRIOR"] = {
		all = { 
		};
		
		-- arms
		[71] = {
			[97462] = { cat=CAT_RAIDDEF; buff=0; };               -- Rallying Cry
			[46968] = { cat=CAT_AOESTUN; talent = 21; buff=0; };  -- Shockwave 
		};
		
		-- fury
		[72] = {
			[97462] = { cat=CAT_RAIDDEF; buff=0; };               -- Rallying Cry
			[46968] = { cat=CAT_AOESTUN; talent = 21; buff=0; };  -- Shockwave 
		};
		
		[73] = {
			[198304] = { cat=CAT_DEF;  };                         -- Intercept
			[46968]  = { cat=CAT_AOESTUN; talent = 11; buff=0; }; -- Shockwave 
		}
	};
	
}
 
-------------------------------------------------------------------------------
local SpellMap = {
}

do
	for _,c in pairs( SpellData ) do
		for _,spec in pairs( c ) do
			for spellid,spelldata in pairs( spec ) do
				SpellMap[spellid] = spelldata
			end
		end
	end
end
 

-------------------------------------------------------------------------------
function Delleren.SpellData:KnownSpell( id )
	return SpellMap[id]
end

-------------------------------------------------------------------------------
function Delleren.SpellData:NoBuffSpell( id )
	local a = SpellMap[id]
	if not a then return nil end
	return a.buff == 0
end

-------------------------------------------------------------------------------
function Delleren.SpellData:BuffSpell( id )
	local a = SpellMap[id]
	if not a then return nil end
	return a.buff ~= 0
end

-------------------------------------------------------------------------------
function Delleren.SpellData:BuildNameMap()
	if self.namemap then return end
	
	self.namemap = {}
	
	for id,data in pairs( SpellMap ) do
		local name = GetSpellInfo( id )
		if name then
			self.namemap[ string.lower( name ) ] = id
		end
	end
end

-------------------------------------------------------------------------------
function Delleren.SpellData:IDFromName( name )
	self:BuildNameMap()
	
	return self.namemap[ string.lower(name) ]
end

-------------------------------------------------------------------------------
local function ApplySpellMods( data, build )
	if data.mods then
		for _,mod in ipairs( data.mods ) do
			mod( data, build )
		end
	end
end

-------------------------------------------------------------------------------
-- Returns a list of spells for a class with a spec, talents.
--
-- @param cls     Unit class filename.
-- @param spec    Specialization ID.
-- @param talents Talent build, a string of digits. 
--
-- @returns A map of spells that they know.
--          [spellid] = { cd = cd duration, maxcharges = max # of charges }
--
function Delleren.SpellData:GetSpells( cls, spec, talents )
	if not SpellData[cls] then return {} end -- Unknown class!
	
	local build = {
		spec    = spec;
		talents = talents; 
	}
	--print( "TEST1", build.spec, build.talents )
	local state = 1
	
	local spells = {}
	
	for k,v in pairs( SpellData[cls].all or {} ) do
		
		if not v.talent or v.talent and TalentSelected( build, v.talent ) then
			ApplySpellMods( v, build )
			
			local spell = {
				cd = v.cd or ((GetSpellBaseCooldown( k ) or 0)/1000);
				maxcharges = v.charges or 1;
			}
			
			spells[k] = spell
		end
	end
	
	for k,v in pairs( SpellData[cls][spec] or {} ) do
		
		if not v.talent or v.talent and TalentSelected( build, v.talent ) then
			ApplySpellMods( v, build )
			
			local spell = {
				cd = v.cd or ((GetSpellBaseCooldown( k ) or 0)/1000);
				maxcharges = v.charges or 1;
			}
			
			spells[k] = spell
		end
	end
	
	return spells
end
